@@ -25,6 +25,7 @@ import logging
 import os
 import pprint
 import sys
+import traceback
 try:
     import jmespath
     have_jmespath = True
@@ -78,15 +79,24 @@ def main():
         sys.exit(0)
 
 
+def print_exception(exc, debug):
+    limit = 0
+    if debug == 1:
+        limit = -1
+    elif debug > 1:
+        limit = None
+    x = traceback.format_exception(None, exc, exc.__traceback__, limit=limit)
+    print(''.join(x), end='', file=sys.stderr)
+    sys.exit(1)
+
+
 def api_request(kwargs, options):
     try:
         with IotApi(**kwargs) as api:
             request(api, options)
 
     except Exception as e:
-        print('%s: %s' % (e.__class__.__name__, e),
-              file=sys.stderr)
-        sys.exit(1)
+        print_exception(e, options['debug'])
 
 
 async def aioapi_request(kwargs, options):
@@ -95,9 +105,7 @@ async def aioapi_request(kwargs, options):
             await aiorequest(api, options)
 
     except Exception as e:
-        print('%s: %s' % (e.__class__.__name__, e),
-              file=sys.stderr)
-        sys.exit(1)
+        print_exception(e, options['debug'])
 
 
 def request(api, options):
@@ -185,11 +193,30 @@ def request(api, options):
         wrap_obj(options, api.alerts_all, **kwargs)
 
     elif options['tag']:
-        resp = api.tag(query_string=options['query_string_obj'])
+        resp = api.tag(
+            offset=options['offset'],
+            pagelength=options['pagelength'],
+            query_string=options['query_string_obj'])
         print_status('tag', resp)
         print_response(options, resp)
         resp.raise_for_status()
 
+    elif options['profile']:
+        resp = api.profile(
+            query_string=options['query_string_obj'])
+        print_status('profile', resp)
+        print_response(options, resp)
+        resp.raise_for_status()
+
+    elif options['policy']:
+        resp = api.policy(
+            offset=options['offset'],
+            pagelength=options['pagelength'],
+            query_string=options['query_string_obj'])
+        print_status('policy', resp)
+        print_response(options, resp)
+        resp.raise_for_status()
+
     elif options['device-update']:
         resp = api.device_update(
             json=options['json_request_obj'],
@@ -301,11 +328,30 @@ async def aiorequest(api, options):
         await aiowrap_obj(options, api.alerts_all, **kwargs)
 
     elif options['tag']:
-        resp = await api.tag(query_string=options['query_string_obj'])
+        resp = await api.tag(
+            offset=options['offset'],
+            pagelength=options['pagelength'],
+            query_string=options['query_string_obj'])
         print_status('tag', resp)
         await aioprint_response(options, resp)
         resp.raise_for_status()
 
+    elif options['profile']:
+        resp = await api.profile(
+            query_string=options['query_string_obj'])
+        print_status('profile', resp)
+        await aioprint_response(options, resp)
+        resp.raise_for_status()
+
+    elif options['policy']:
+        resp = await api.policy(
+            offset=options['offset'],
+            pagelength=options['pagelength'],
+            query_string=options['query_string_obj'])
+        print_status('policy', resp)
+        await aioprint_response(options, resp)
+        resp.raise_for_status()
+
     elif options['device-update']:
         resp = await api.device_update(
             json=options['json_request_obj'],
@@ -362,7 +408,7 @@ async def aiowrap_obj(options, func, **kwargs):
     async for ok, x in func(**kwargs):
         if not ok:
             print_status(func.__name__, x)
-            aioprint_response(options, x)
+            await aioprint_response(options, x)
             x.raise_for_status()
         elif wrap:
             obj['things'].append(x)
@@ -525,6 +571,8 @@ def parse_opts():
         'alert': False,
         'alerts': False,
         'tag': False,
+        'profile': False,
+        'policy': False,
         'offset': None,
         'pagelength': None,
         'device-update': False,
@@ -555,7 +603,7 @@ def parse_opts():
         'deviceid=', 'ip=',
         'vuln', 'groupby=', 'vulns',
         'alert', 'alerts',
-        'tag',
+        'tag', 'profile', 'policy',
         'offset=', 'pagelength=',
         'device-update', 'vuln-update', 'alert-update',
         'id=',
@@ -616,6 +664,10 @@ def parse_opts():
             options['alerts'] = True
         elif opt == '--tag':
             options['tag'] = True
+        elif opt == '--profile':
+            options['profile'] = True
+        elif opt == '--policy':
+            options['policy'] = True
         elif opt == '--offset':
             options['offset'] = arg
         elif opt == '--pagelength':
@@ -758,6 +810,8 @@ def usage():
     --alert                  get security alert API request
     --alerts                 get all security alerts
     --tag                    get tag API request
+    --profile                get profile mapping API request
+    --policy                 get policy recommendation API request
     --offset num             items offset
     --pagelength num         number of items to return
     --device-update          update device API request
